@@ -1,8 +1,12 @@
+from __future__ import absolute_import
 import warnings
 
 from .api import _, is_validator, FancyValidator, Invalid, NoDefault
 from . import declarative
 from .exc import FERuntimeWarning
+import six
+from six.moves import map
+from six.moves import zip
 
 __all__ = ['Schema']
 
@@ -81,7 +85,7 @@ class Schema(FancyValidator):
         # Scan through the class variables we've defined *just*
         # for this subclass, looking for validators (both classes
         # and instances):
-        for key, value in new_attrs.items():
+        for key, value in six.iteritems(new_attrs):
             if key in ('pre_validators', 'chained_validators'):
                 if is_validator(value):
                     msg = "Any validator with the name %s will be ignored." % \
@@ -96,12 +100,12 @@ class Schema(FancyValidator):
             elif key in cls.fields:
                 del cls.fields[key]
 
-        for name, value in cls.fields.items():
+        for name, value in six.iteritems(cls.fields):
             cls.add_field(name, value)
 
     def __initargs__(self, new_attrs):
         self.fields = self.fields.copy()
-        for key, value in new_attrs.items():
+        for key, value in six.iteritems(new_attrs):
             if key in ('pre_validators', 'chained_validators'):
                 if is_validator(value):
                     msg = "Any validator with the name %s will be ignored." % \
@@ -246,7 +250,7 @@ class Schema(FancyValidator):
             state.full_dict = value_dict
         try:
             __traceback_info__ = None
-            for name, value in value_dict.items():
+            for name, value in six.iteritems(value_dict):
                 __traceback_info__ = 'for_python in %s' % name
                 try:
                     unused.remove(name)
@@ -327,7 +331,7 @@ class Schema(FancyValidator):
         result = []
         result.extend(self.pre_validators)
         result.extend(self.chained_validators)
-        result.extend(iter(self.fields.values()))
+        result.extend(six.itervalues(self.fields))
         return result
 
     def is_empty(self, value):
@@ -338,7 +342,7 @@ class Schema(FancyValidator):
         return {}
 
     def _value_is_iterator(self, value):
-        if isinstance(value, str):
+        if isinstance(value, six.string_types + (six.binary_type, )):
             return False
         elif isinstance(value, (list, tuple)):
             return True
@@ -361,16 +365,16 @@ def format_compound_error(v, indent=0):
             # version if possible, and unicode() if necessary, because
             # testing for the presence of a __unicode__ method isn't
             # enough
-            return str(v)
+            return six.text_type(v)
     elif isinstance(v, dict):
         return ('%s\n' % (' ' * indent)).join(
             '%s: %s' % (k, format_compound_error(value, indent=len(k) + 2))
-            for k, value in sorted(v.items()) if value is not None)
+            for k, value in sorted(six.iteritems(v)) if value is not None)
     elif isinstance(v, list):
         return ('%s\n' % (' ' * indent)).join(
             '%s' % (format_compound_error(value, indent=indent))
             for value in v if value is not None)
-    elif isinstance(v, str):
+    elif isinstance(v, six.string_types):
         return v
     else:
         assert False, "I didn't expect something like %s" % repr(v)
@@ -383,7 +387,7 @@ def merge_dicts(d1, d2):
 
 
 def merge_values(v1, v2):
-    if isinstance(v1, str) and isinstance(v2, str):
+    if isinstance(v1, six.string_types) and isinstance(v2, six.string_types):
         return v1 + '\n' + v2
     elif isinstance(v1, (list, tuple)) and isinstance(v2, (list, tuple)):
         return merge_lists(v1, v2)
@@ -474,7 +478,7 @@ class SimpleFormValidator(FancyValidator):
         errors = self.func(value_dict, state, self)
         if not errors:
             return value_dict
-        if isinstance(errors, str):
+        if isinstance(errors, six.string_types):
             raise Invalid(errors, value_dict, state)
         elif isinstance(errors, dict):
             raise Invalid(
