@@ -1,8 +1,10 @@
 # -*- coding: utf-8 -*-
+from __future__ import absolute_import
+from __future__ import unicode_literals
 
 import datetime
 import unittest
-from nose.plugins.skip import SkipTest
+import pytest
 
 from formencode import validators
 from formencode.validators import Invalid
@@ -10,6 +12,10 @@ from formencode.variabledecode import NestedVariables
 from formencode.schema import Schema
 from formencode.foreach import ForEach
 from formencode.api import NoDefault
+import six
+
+if not six.PY2:
+    unicode = str
 
 
 def validate(validator, value):
@@ -49,7 +55,7 @@ class TestByteStringValidator(unittest.TestCase):
         self.messages = self.validator.message
 
     def test_alias(self):
-        if str is not str:  # Python 2
+        if str is not six.text_type:  # Python 2
             self.assertTrue(self.validator is validators.String)
 
     def test_docstring(self):
@@ -87,7 +93,7 @@ class TestUnicodeStringValidator(unittest.TestCase):
         self.validator = validators.UnicodeString
 
     def test_alias(self):
-        if str is str:  # Python 3
+        if str is six.text_type:  # Python 3
             self.assertTrue(self.validator is validators.String)
 
     def test_docstring(self):
@@ -97,7 +103,7 @@ class TestUnicodeStringValidator(unittest.TestCase):
     def test_unicode(self):
         un = self.validator()
         self.assertEqual(un.to_python(12), '12')
-        self.assertTrue(type(un.to_python(12)) is str)
+        self.assertTrue(type(un.to_python(12)) is six.text_type)
         self.assertEqual(un.from_python(12), '12'.encode('ascii'))
         self.assertTrue(type(un.from_python(12)) is bytes)
 
@@ -106,37 +112,37 @@ class TestUnicodeStringValidator(unittest.TestCase):
         us = 'käse'
         u7s, u8s = us.encode('utf-7'), us.encode('utf-8')
         self.assertEqual(uv.to_python(u8s), us)
-        self.assertTrue(type(uv.to_python(u8s)) is str)
+        self.assertTrue(type(uv.to_python(u8s)) is six.text_type)
         self.assertEqual(uv.from_python(us), u8s)
         self.assertTrue(type(uv.from_python(us)) is bytes)
         uv = self.validator(encoding='utf-7')
         self.assertEqual(uv.to_python(u7s), us)
-        self.assertTrue(type(uv.to_python(u7s)) is str)
+        self.assertTrue(type(uv.to_python(u7s)) is six.text_type)
         self.assertEqual(uv.from_python(us), u7s)
         self.assertTrue(type(uv.from_python(us)) is bytes)
         uv = self.validator(inputEncoding='utf-7')
         self.assertEqual(uv.to_python(u7s), us)
-        self.assertTrue(type(uv.to_python(u7s)) is str)
+        self.assertTrue(type(uv.to_python(u7s)) is six.text_type)
         uv = self.validator(outputEncoding='utf-7')
         self.assertEqual(uv.from_python(us), u7s)
         self.assertTrue(type(uv.from_python(us)) is bytes)
         uv = self.validator(inputEncoding=None)
         self.assertEqual(uv.to_python(us), us)
-        self.assertTrue(type(uv.to_python(us)) is str)
+        self.assertTrue(type(uv.to_python(us)) is six.text_type)
         self.assertEqual(uv.from_python(us), u8s)
         self.assertTrue(type(uv.from_python(us)) is bytes)
         uv = self.validator(outputEncoding=None)
         self.assertEqual(uv.to_python(u8s), us)
-        self.assertTrue(type(uv.to_python(u8s)) is str)
+        self.assertTrue(type(uv.to_python(u8s)) is six.text_type)
         self.assertEqual(uv.from_python(us), us)
-        self.assertTrue(type(uv.from_python(us)) is str)
+        self.assertTrue(type(uv.from_python(us)) is six.text_type)
 
         def test_unicode_empty(self):
             iv = self.validator()
-            for value in [None, "", ""]:
+            for value in [None, b"", ""]:
                 result = iv.to_python(value)
                 self.assertEqual(result, "")
-                self.assertTrue(isinstance(result, str))
+                self.assertTrue(isinstance(result, six.text_type))
 
 
 class TestIntValidator(unittest.TestCase):
@@ -192,7 +198,7 @@ class TestNumberValidator(unittest.TestCase):
         try:
             inf = float('infinity')
         except ValueError:
-            raise SkipTest
+            pytest.skip('skipping incompatible float test')
         self.assertEqual(self.validator.to_python('infinity'), inf)
 
 
@@ -207,14 +213,14 @@ class TestDateConverterValidator(unittest.TestCase):
             dc.to_python('20/12/150')
         except Invalid as e:
             self.assertTrue(
-                'Please enter a four-digit year after 1899' in str(e))
+                'Please enter a four-digit year after 1899' in unicode(e))
         else:
             self.fail('Date should be invalid')
         try:
             dc.to_python('oh/happy/day')
         except Invalid as e:
             self.assertTrue(
-                'Please enter the date in the form DD/MM/YYYY' in str(e))
+                'Please enter the date in the form DD/MM/YYYY' in unicode(e))
         else:
             self.fail('Date should be invalid')
 
@@ -234,14 +240,14 @@ class TestDateConverterValidator(unittest.TestCase):
             dc = self.validator(month_style='Klingon')
         except TypeError as e:
             self.assertTrue(
-                str(e) == "Bad month_style: 'klingon'")
+                unicode(e).replace("u'", "'") == "Bad month_style: 'klingon'")
         else:
             self.fail('month_style should be invalid')
         try:
             dc = self.validator(month_style='ydm')
         except TypeError as e:
             self.assertTrue(
-                str(e) == "Bad month_style: 'ydm'")
+                unicode(e).replace("u'", "'") == "Bad month_style: 'ydm'")
         else:
             self.fail('month_style should be invalid')
 
@@ -255,14 +261,14 @@ class TestDateConverterValidator(unittest.TestCase):
         try:
             self.assertEqual(dc.to_python('20/12/2007'), d)
         except Invalid as e:
-            self.assertTrue('Please enter a month from 1 to 12' in str(e))
+            self.assertTrue('Please enter a month from 1 to 12' in unicode(e))
         else:
             self.fail('Date should be invalid')
         try:
             self.assertEqual(dc.to_python('12/Dec/2007'), d)
         except Invalid as e:
             self.assertTrue(
-                'Please enter the date in the form MM/DD/YYYY' in str(e))
+                'Please enter the date in the form MM/DD/YYYY' in unicode(e))
         else:
             self.fail('Date should be invalid')
 
@@ -276,14 +282,14 @@ class TestDateConverterValidator(unittest.TestCase):
         try:
             self.assertEqual(dc.to_python('12/20/2007'), d)
         except Invalid as e:
-            self.assertTrue('Please enter a month from 1 to 12' in str(e))
+            self.assertTrue('Please enter a month from 1 to 12' in unicode(e))
         else:
             self.fail('Date should be invalid')
         try:
             self.assertEqual(dc.to_python('Dec/12/2007'), d)
         except Invalid as e:
             self.assertTrue(
-                'Please enter the date in the form DD/MM/YYYY' in str(e))
+                'Please enter the date in the form DD/MM/YYYY' in unicode(e))
         else:
             self.fail('Date should be invalid')
 
@@ -297,14 +303,14 @@ class TestDateConverterValidator(unittest.TestCase):
         try:
             self.assertEqual(dc.to_python('2013/30/06'), d)
         except Invalid as e:
-            self.assertTrue('Please enter a month from 1 to 12' in str(e))
+            self.assertTrue('Please enter a month from 1 to 12' in unicode(e))
         else:
             self.fail('Date should be invalid')
         try:
             self.assertEqual(dc.to_python('2013/06/Jun'), d)
         except Invalid as e:
             self.assertTrue(
-                'Please enter the date in the form YYYY/MM/DD' in str(e))
+                'Please enter the date in the form YYYY/MM/DD' in unicode(e))
         else:
             self.fail('Date should be invalid')
 
@@ -318,21 +324,21 @@ class TestDateConverterValidator(unittest.TestCase):
         try:
             self.assertEqual(dc.to_python('20/2007'), d)
         except Invalid as e:
-            self.assertTrue('Please enter a month from 1 to 12' in str(e))
+            self.assertTrue('Please enter a month from 1 to 12' in unicode(e))
         else:
             self.fail('Date should be invalid')
         try:
             self.assertEqual(dc.to_python('12/20/2007'), d)
         except Invalid as e:
             self.assertTrue(
-                'Please enter the date in the form MM/YYYY' in str(e))
+                'Please enter the date in the form MM/YYYY' in unicode(e))
         else:
             self.fail('Date should be invalid')
         try:
             self.assertEqual(dc.to_python('2007/Dec'), d)
         except Invalid as e:
             self.assertTrue(
-                'Please enter the date in the form MM/YYYY' in str(e))
+                'Please enter the date in the form MM/YYYY' in unicode(e))
         else:
             self.fail('Date should be invalid')
 
@@ -383,29 +389,29 @@ class TestTimeConverterValidator(unittest.TestCase):
             tc.to_python('25:30:15')
         except Invalid as e:
             self.assertTrue(
-                'You must enter an hour in the range 0-23' in str(e))
+                'You must enter an hour in the range 0-23' in unicode(e))
         else:
             self.fail('Time should be invalid')
         try:
             tc.to_python('20:75:15')
         except Invalid as e:
             self.assertTrue(
-                'You must enter a minute in the range 0-59' in str(e))
+                'You must enter a minute in the range 0-59' in unicode(e))
         else:
             self.fail('Time should be invalid')
         try:
             tc.to_python('20:30:75')
         except Invalid as e:
             self.assertTrue(
-                'You must enter a second in the range 0-59' in str(e))
+                'You must enter a second in the range 0-59' in unicode(e))
         else:
             self.fail('Time should be invalid')
         try:
             tc.to_python('20:30:zx')
         except Invalid as e:
             self.assertTrue(
-                'The second value you gave is not a number' in str(e))
-            self.assertTrue('zx' in str(e))
+                'The second value you gave is not a number' in unicode(e))
+            self.assertTrue('zx' in unicode(e))
         else:
             self.fail('Time should be invalid')
 
@@ -636,6 +642,36 @@ class TestOneOfValidator(unittest.TestCase):
 
         self.assertEqual(expected, value)
 
+    def test_mixed_list(self):
+        """before `_message_vars_decode`, the errors were:
+            Value must be one of: a; b; ö; ™; 1; 2; 3 (not u'c')
+            Value must be one of: a; b; ö; ™; 1; 2; 3 (not u'\xa7')
+        """
+        o = validators.OneOf(['a', 'b', 'ö', '™', 1, 2, 3])
+        try:
+            o.to_python('c')
+            raise ValueError("exception expected")
+        except Invalid as e:
+            self.assertTrue("Value must be one of: a; b; ö; ™; 1; 2; 3 (not 'c')" in unicode(e))
+        try:
+            o.to_python(9)
+            raise ValueError("exception expected")
+        except Invalid as e:
+            self.assertTrue("Value must be one of: a; b; ö; ™; 1; 2; 3 (not 9)" in unicode(e))
+        try:
+            o.to_python(u'd')
+            raise ValueError("exception expected")
+        except Invalid as e:
+            self.assertTrue("Value must be one of: a; b; ö; ™; 1; 2; 3 (not 'd')" in unicode(e))
+        try:
+            o.to_python('§')
+            raise ValueError("exception expected")
+        except Invalid as e:
+            if six.PY2:
+                self.assertTrue(r"Value must be one of: a; b; ö; ™; 1; 2; 3 (not u'\xa7')" in unicode(e))
+            else:
+                self.assertTrue(r"Value must be one of: a; b; ö; ™; 1; 2; 3 (not '§')" in unicode(e))
+
 
 class TestIPAddressValidator(unittest.TestCase):
 
@@ -659,13 +695,13 @@ class TestIPAddressValidator(unittest.TestCase):
         try:
             validate('1.2.3.037')
         except Invalid as e:
-            self.assertTrue('The octets must not have leading zeros' in str(e))
+            self.assertTrue('The octets must not have leading zeros' in unicode(e))
         else:
             self.fail('IP address octets with leading zeros should be invalid')
         try:
             validate('1.2.3.0377')
         except Invalid as e:
-            self.assertTrue('The octets must not have leading zeros' in str(e))
+            self.assertTrue('The octets must not have leading zeros' in unicode(e))
         else:
             self.fail('IP octets with leading zeros should be invalid')
 
@@ -679,7 +715,7 @@ class TestIPAddressValidator(unittest.TestCase):
             validate('1.2.3.0377')
         except Invalid as e:
             self.assertTrue("The octets must be within the range of 0-255"
-                " (not '377')" in str(e))
+                " (not '377')" in unicode(e).replace("u'", "'"))
         else:
             self.fail(
                 'IP address octets should not be interpreted as octal numbers')
@@ -737,6 +773,7 @@ class TestRequireIfMissingValidator(unittest.TestCase):
             validate(v, dict(operator='', operand=0)),
             dict(operator='Please enter a value'))
 
+
 class TestRequireIfMatchingValidator(unittest.TestCase):
 
     def setUp(self):
